﻿using DockingAdapterMVVM;
using Microsoft.Win32;
using Syncfusion.Windows.Shared;
using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.ComponentModel;
using System.IO;
using System.Linq;
using System.Reflection;
using System.Text;
using System.Threading.Tasks;
using System.Windows;

namespace DockingManagerMVVM
{
    public class ViewModel : NotificationObject
    {
        private ObservableCollection<IDockElement> documents;

        public ObservableCollection<IDockElement> Documents
        {
            get { return documents; }
            set { documents = value; }
        }

        private ObservableCollection<Workspace> workspaces;

        public ObservableCollection<Workspace> Workspaces
        {
            get { return workspaces; }
            set { workspaces = value; }
        }

        private ObservableCollection<CustomRibbonTab> _customTabs;

        public ObservableCollection<CustomRibbonTab> CustomTabs
        {
            get { return _customTabs; }
            set { _customTabs = value; }
        }
        
        
        private Document activeDocument;

        public Document ActiveDocument
        {
            get { return activeDocument; }
            set { activeDocument = value; RaisePropertyChanged("ActiveDocument"); }
        }

        private PropertiesViewModel propertiesViewModel;

        public PropertiesViewModel PropertiesViewModel
        {
            get { return propertiesViewModel; }
            set { propertiesViewModel = value; }
        }

        private DocumentsViewModel documentsViewModel;

        public DocumentsViewModel DocumentsViewModel
        {
            get { return documentsViewModel; }
            set { documentsViewModel = value; }
        }


        public ViewModel()
        {
            Workspaces = new ObservableCollection<Workspace>();
            CustomTabs = new ObservableCollection<CustomRibbonTab>();
            string exepath = Assembly.GetExecutingAssembly().Location;

            if (!DesignerProperties.GetIsInDesignMode(new DependencyObject()))
            {
                Workspaces.Add(new Document() { Path = Path.GetDirectoryName(exepath) + @"\Samples\Sample 1.txt", Header = "Sample 1.txt" });
                Workspaces.Add(new Document() { Header = "Sample 2.txt", Path = Path.GetDirectoryName(exepath) + @"\Samples\Sample 2.txt" });
                Workspaces.Add(new Document() { Header = "Sample 3.txt", Path = Path.GetDirectoryName(exepath) + @"\Samples\Sample 3.txt" });
            }

            PropertiesViewModel = new DockingManagerMVVM.PropertiesViewModel() { ViewModel = this };
            DocumentsViewModel = new DockingManagerMVVM.DocumentsViewModel() { ViewModel = this };

            PopulateRibbonItems();

            Workspaces.Add(DocumentsViewModel);
            Workspaces.Add(PropertiesViewModel);
        }

        private void PopulateRibbonItems()
        {
            //RibbonTab
            CustomRibbonTab hometab = new CustomRibbonTab() { Caption = "Home" };
            //RibbonBar
            CustomRibbonBar homebar1 = new CustomRibbonBar() { Header = "Clipboard" };
            CustomRibbonBar homebar2 = new CustomRibbonBar() { Header = "Font" };
            //RibbonItem
            CustomRibbonItem homebar1item1 = new CustomRibbonItem() { Label = "Paste", IsLarge = true, IsSplitButton=true };
            CustomRibbonItem homebar1item2 = new CustomRibbonItem() { Label = "Cut"};
            CustomRibbonItem homebar1item3 = new CustomRibbonItem() { Label = "Copy"};

            CustomRibbonItem homebar2item1 = new CustomRibbonItem() { Label = "Bold" };
            CustomRibbonItem homebar2item2 = new CustomRibbonItem() { Label = "Italic" };
            CustomRibbonItem homebar2item3 = new CustomRibbonItem() { Label = "Underline" };
            homebar1.RibbonItems.Add(homebar1item1);
            homebar1.RibbonItems.Add(homebar1item2);
            homebar1.RibbonItems.Add(homebar1item3);

            homebar2.RibbonItems.Add(homebar2item1);
            homebar2.RibbonItems.Add(homebar2item2);
            homebar2.RibbonItems.Add(homebar2item3);

            hometab.RibbonBars.Add(homebar1);
            hometab.RibbonBars.Add(homebar2);
            //RibbonTab
            CustomRibbonTab inserttab = new CustomRibbonTab() { Caption = "Insert" };
            //RibbonBar
            CustomRibbonBar insertbar1 = new CustomRibbonBar() { Header = "Table" };
            CustomRibbonBar insertbar2 = new CustomRibbonBar() { Header = "Mail" };
            //RibbonItem
            CustomRibbonItem insertbar1item1 = new CustomRibbonItem() { Label = "Tables", IsLarge = true, IsSplitButton = true };

            CustomRibbonItem insertbar2item1 = new CustomRibbonItem() { Label = "Attach Files" };
            CustomRibbonItem insertbar2item2 = new CustomRibbonItem() { Label = "Business Card" };
            CustomRibbonItem insertbar2item3 = new CustomRibbonItem() { Label = "Audio" };
            insertbar1.RibbonItems.Add(insertbar1item1);
            insertbar2.RibbonItems.Add(insertbar2item1);
            insertbar2.RibbonItems.Add(insertbar2item2);
            insertbar2.RibbonItems.Add(insertbar2item3);
            inserttab.RibbonBars.Add(insertbar1);
            inserttab.RibbonBars.Add(insertbar2);

            //RibbonTab
            CustomRibbonTab edittab = new CustomRibbonTab() { Caption = "Edit" };
            //RibbonBar
            CustomRibbonBar editbar = new CustomRibbonBar() { Header = "Editing" };
            //RibbonItem
            CustomRibbonItem editbaritem = new CustomRibbonItem() { Label = "Replace", IsLarge=true };
            editbar.RibbonItems.Add(editbaritem);
            edittab.RibbonBars.Add(editbar);


            CustomTabs.Add(hometab);
            CustomTabs.Add(inserttab);
            CustomTabs.Add(edittab);
        }

        private void OpenDocument()
        {
            OpenFileDialog dialog = new OpenFileDialog();
            dialog.Filter = "Text files (*.txt)|*.txt";
            if (dialog.ShowDialog().Value)
            {
                Document document = new Document() { Path = dialog.FileName, Header = dialog.SafeFileName };
                Workspaces.Add(document);
                ActiveDocument = document;
                DocumentsViewModel.RaiseDocumentsPropertyChanged();
            }
        }

        private void Exit()
        {
            Application.Current.MainWindow.Close();
        }

        private DelegateCommand<object> openDocumentCommand;

        public DelegateCommand<object> OpenDocumentCommand
        {
            get
            {
                if (openDocumentCommand == null)
                {
                    openDocumentCommand = new DelegateCommand<object>(param => OpenDocument());
                }
                return openDocumentCommand;
            }
        }

        private DelegateCommand<object> closeCommand;

        public DelegateCommand<object> CloseCommand
        {
            get
            {
                if (closeCommand == null)
                {
                    closeCommand = new DelegateCommand<object>(param => Exit());
                }
                return closeCommand;
            }
        }
    }
}
